##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../screen.4gs/r_menu.4gl  1.1  Last Delta: 10/26/88
##############################################################################

#######################################################################
#   ring_menu --- generic menu interface
#   - all data entry checking (including duplicate data when a "unique"
#     index is specified) has to be done in llh_input()
#   - note that ring_rowid, ring_cursor, and ring_total are not global,
#     but passed to functions as needed.
#   - the functions that ring_menu call can be substituted by routines in
#     local code having the same name. (if ring_menu is in a library)
#   - if the ring_rowid is non-zero, then the p_* variables are set.
#   - if ring_total is non-zero then the cursor is open.
#   - all status/interrupt handling is done by the ring_* functions so
#     the low level functions don't have to. (unless they want to)
#     - if status/interrupt handling is done by low level and the variables
#       are reset to 0, then upper level can't tell if an error occured.
#   - ring_menu relies on several low level routines to be present:
#     the ring_menu functions handle all of the error/interrupt trapping
#       llh_display()
#        - fills the screen variables with the program variables. (p_* --> s_*)
#       llh_input(type)
#        - this routine calls the input statement for screen->program vars.
#        - type is a char(6) passed as "add", or "update"
#        - if "add" mode, then you should initialize the p_record
#        - initialize uses the maps defaults, not the forms defaults.  if you
#          need to set a default that isn't specified in the map, you must
#          use "let" after the initialize statement
#        - this function does none of the writing to disk.
#       llh_add() inserts a new record into the database after an "add"
#          function.
#       mlh_validate()
#        - low level validation routine called before all disk writes
#       llh_read(ring_rowid)
#        - this function opens a cursor for update selecting just the
#          record defined by rowid.
#        - it also sets all of the p_* variables.
#        - because it fetches the record with an update cursor, the record
#          is locked.
#       llh_update(ring_rowid)
#        - this only works after an llh_read that defined a cursor for update.
#        - uses the update cursor defined in llh_read
#          to write the record out from p_* to the current cursor
#        - it must also close the cursor to unlock the row.
#       llh_delete()
#        - this only works after an llh_read that defined a cursor for update.
#        - uses the update cursor defined in llh_read
#          to delete the record.
#        - it must also close the cursor to unlock the row.
#       mlh_cursor("function")
#        - these routines work on the browse cursor (non-update cursor)
#        - Functions include:
#        define      define the cursor. this includes constructing the
#                    select statement, and opening the cursor.
#        first       position the cursor on the first row.
#        last        position the cursor on the last row.
#        next        position the cursor on the next row.
#        prev        position the cursor on the previous row.
#        (number)    position the cursor on the row corresponding to the number.
#        fetch       fetch the current row that the cursor is positioned at
#                    into the p_* variables.
#        close       close the cursor.
#######################################################################


#######################################################################
function ring_menu()
#######################################################################
#
    define
        ring_rowid  integer, # the rowid of the current screen record
                             # or -1 if there is no record on the screen
        ring_cursor integer, # pointer to the absolute cursor address.
                             # undefined for a closed cursor
        ring_total  integer  # total number of elements in the current cursor.
                             # if ring_total > 0, then the cursor is open.
                             # if ring_total = 0, then the cursor is closed.
    # if ring_cursor is 0 and ring_total > 0, then there is a open cursor, but
    # the record on the screen doesn't have anything to do with it.

    # ring_border() with -1 arguments means to draw the first border
    call ring_border(-1, -1, -1)

	# load all elements into the cursor...
	call ring_define("1=1")
		returning ring_rowid, ring_cursor, ring_total

    call ring_border(ring_rowid, ring_cursor, ring_total)
    menu " Action"
        command key (a,F1)
            "Add" " Create a new document"
            call ring_add(ring_rowid, ring_cursor, ring_total)
				returning ring_rowid, ring_cursor, ring_total
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Update" " Change this document"
            call ring_update(ring_rowid)
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (d,F2)
            "Delete" " Erase this document"
            call ring_delete(ring_rowid) returning ring_rowid
            if ring_rowid = -1 and ring_cursor != 0 and ring_total > 0
            then
                let ring_rowid = 0      # deleted cursor element
            end if
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Find" " Select a document, or group of documents"
            call ring_find(ring_rowid,ring_cursor,ring_total)
                returning ring_rowid, ring_cursor, ring_total
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Browse" " Page through selected documents"
            call ring_browse(ring_rowid,ring_cursor,ring_total)
                returning ring_rowid, ring_cursor, ring_total
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (n,F3)
			"Nxt" " View next document"
            if ring_total = 0
            then
                continue menu
            end if
            if ring_cursor = ring_total
            then
                let ring_cursor = 0
            end if
            let ring_cursor = ring_cursor + 1
            call ring_view(ring_cursor) returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (p,F4)
			"Prv" " View previous document"
            if ring_total = 0
            then
                continue menu
            end if
            if ring_cursor <= 1
            then
                let ring_cursor = ring_total + 1
            end if
            let ring_cursor = ring_cursor - 1
            call ring_view(ring_cursor) returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (control-i,t)
			"Tab" " Access document detail lines"
			call ring_clear()
			display "" at 22,1
			display "Scroll:  [TAB], [DEL], or [ESC]  to Quit"
				at 1,2
			display "[DOWN ARROW] or [UP ARROW] to Scroll,  [F3] or [F4]  to Page"
				at 2,2
            call mld_scroll()
			let int_flag = 0
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Options" " Additional Options"
            call ring_options(ring_rowid,ring_cursor,ring_total)
                    returning ring_rowid, ring_cursor, ring_total
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (e,x,q)
            "Quit" " End the program"
            if ok_exit()
			then
				clear screen
            	exit menu
			end if
        command key ("!")
            call ring_bang()
            call ring_border(ring_rowid, ring_cursor, ring_total)
    end menu
    return
end function
# ring_menu()
